{
  This file was automatically generated on 2019-08-23.

  Delphi/Lazarus Bindings Version 2.1.25

  If you have a bugfix for this file and want to commit it,
  please fix the bug in the generator. You can find a link
  to the generators git on tinkerforge.com
}

unit BrickHAT;

{$ifdef FPC}{$mode OBJFPC}{$H+}{$endif}

interface

uses
  Device, IPConnection, LEConverter, Math, SyncObjs;

const
  BRICK_HAT_DEVICE_IDENTIFIER = 111;
  BRICK_HAT_DEVICE_DISPLAY_NAME = 'HAT Brick';

  BRICK_HAT_FUNCTION_SET_SLEEP_MODE = 1;
  BRICK_HAT_FUNCTION_GET_SLEEP_MODE = 2;
  BRICK_HAT_FUNCTION_SET_BRICKLET_POWER = 3;
  BRICK_HAT_FUNCTION_GET_BRICKLET_POWER = 4;
  BRICK_HAT_FUNCTION_GET_VOLTAGES = 5;
  BRICK_HAT_FUNCTION_GET_SPITFP_ERROR_COUNT = 234;
  BRICK_HAT_FUNCTION_SET_BOOTLOADER_MODE = 235;
  BRICK_HAT_FUNCTION_GET_BOOTLOADER_MODE = 236;
  BRICK_HAT_FUNCTION_SET_WRITE_FIRMWARE_POINTER = 237;
  BRICK_HAT_FUNCTION_WRITE_FIRMWARE = 238;
  BRICK_HAT_FUNCTION_SET_STATUS_LED_CONFIG = 239;
  BRICK_HAT_FUNCTION_GET_STATUS_LED_CONFIG = 240;
  BRICK_HAT_FUNCTION_GET_CHIP_TEMPERATURE = 242;
  BRICK_HAT_FUNCTION_RESET = 243;
  BRICK_HAT_FUNCTION_WRITE_UID = 248;
  BRICK_HAT_FUNCTION_READ_UID = 249;
  BRICK_HAT_FUNCTION_GET_IDENTITY = 255;


  BRICK_HAT_BOOTLOADER_MODE_BOOTLOADER = 0;
  BRICK_HAT_BOOTLOADER_MODE_FIRMWARE = 1;
  BRICK_HAT_BOOTLOADER_MODE_BOOTLOADER_WAIT_FOR_REBOOT = 2;
  BRICK_HAT_BOOTLOADER_MODE_FIRMWARE_WAIT_FOR_REBOOT = 3;
  BRICK_HAT_BOOTLOADER_MODE_FIRMWARE_WAIT_FOR_ERASE_AND_REBOOT = 4;
  BRICK_HAT_BOOTLOADER_STATUS_OK = 0;
  BRICK_HAT_BOOTLOADER_STATUS_INVALID_MODE = 1;
  BRICK_HAT_BOOTLOADER_STATUS_NO_CHANGE = 2;
  BRICK_HAT_BOOTLOADER_STATUS_ENTRY_FUNCTION_NOT_PRESENT = 3;
  BRICK_HAT_BOOTLOADER_STATUS_DEVICE_IDENTIFIER_INCORRECT = 4;
  BRICK_HAT_BOOTLOADER_STATUS_CRC_MISMATCH = 5;
  BRICK_HAT_STATUS_LED_CONFIG_OFF = 0;
  BRICK_HAT_STATUS_LED_CONFIG_ON = 1;
  BRICK_HAT_STATUS_LED_CONFIG_SHOW_HEARTBEAT = 2;
  BRICK_HAT_STATUS_LED_CONFIG_SHOW_STATUS = 3;

type
  TArray0To2OfUInt8 = array [0..2] of byte;
  TArray0To63OfUInt8 = array [0..63] of byte;

  /// <summary>
  ///  HAT for Raspberry Pi with 8 Bricklets ports and real-time clock
  /// </summary>
  TBrickHAT = class(TDevice)
  private
  protected
  public
    /// <summary>
    ///  Creates an object with the unique device ID <c>uid</c>. This object can
    ///  then be added to the IP connection.
    /// </summary>
    constructor Create(const uid__: string; ipcon_: TIPConnection);

    /// <summary>
    ///  Sets the sleep mode.
    ///  
    ///  <note>
    ///   Calling this function will cut the Raspberry Pi's power after Power Off Delay seconds.
    ///   You have to shut down the Operating System yourself, e.g. by calling 'sudo shutdown -h now'.
    ///  </note>
    ///  
    ///  Parameters:
    ///  
    ///  * Power Off Delay: Time in seconds before the RPi/Bricklets are powered off.
    ///  * Power Off Duration: Duration in seconds that the RPi/Bricklets stay powered off.
    ///  * Raspberry Pi Off: RPi is powered off if set to true.
    ///  * Bricklets Off: Bricklets are powered off if set to true.
    ///  * Enable Sleep Indicator: If set to true, the status LED will blink in a 1s interval
    ///    during the whole power off duration. This will draw additional 0.3mA.
    ///  
    ///  Example: To turn RPi and Bricklets off in 5 seconds for 10 minutes with sleep
    ///  indicator enabled, call (5, 60*10, true, true, true).
    ///  
    ///  This function can also be used to implement a watchdog. To do this you can
    ///  write a program that calls this function once per second in a loop with
    ///  (10, 2, true, false, false). If the RPi crashes or gets stuck
    ///  the HAT will reset the RPi after 10 seconds.
    /// </summary>
    procedure SetSleepMode(const powerOffDelay: longword; const powerOffDuration: longword; const raspberryPiOff: boolean; const brickletsOff: boolean; const enableSleepIndicator: boolean); virtual;

    /// <summary>
    ///  Returns the sleep mode settings as set by <see cref="BrickHAT.TBrickHAT.SetSleepMode"/>.
    /// </summary>
    procedure GetSleepMode(out powerOffDelay: longword; out powerOffDuration: longword; out raspberryPiOff: boolean; out brickletsOff: boolean; out enableSleepIndicator: boolean); virtual;

    /// <summary>
    ///  Set to true/false to turn the power supply of the connected Bricklets on/off.
    ///  
    ///  By default the Bricklets are on.
    /// </summary>
    procedure SetBrickletPower(const brickletPower: boolean); virtual;

    /// <summary>
    ///  Returns the power status of the connected Bricklets as set by <see cref="BrickHAT.TBrickHAT.SetBrickletPower"/>.
    /// </summary>
    function GetBrickletPower: boolean; virtual;

    /// <summary>
    ///  Returns the USB supply voltage and the DC input supply voltage in mV.
    ///  
    ///  There are three possible combinations:
    ///  
    ///  * Only USB connected: The USB supply voltage will be fed back to the
    ///    DC input connector. You will read the USB voltage and a slightly lower
    ///    voltage on the DC input.
    ///  * Only DC input connected: The DC voltage will not be fed back to the
    ///    USB connector. You will read the DC input voltage and the USB voltage
    ///    will be 0.
    ///  * USB and DC input connected: You will read both voltages. In this case
    ///    the USB supply will be without load, but it will work as backup if you
    ///    disconnect the DC input (or if the DC input voltage falls below the
    ///    USB voltage).
    /// </summary>
    procedure GetVoltages(out voltageUSB: word; out voltageDC: word); virtual;

    /// <summary>
    ///  Returns the error count for the communication between Brick and Bricklet.
    ///  
    ///  The errors are divided into
    ///  
    ///  * ACK checksum errors,
    ///  * message checksum errors,
    ///  * framing errors and
    ///  * overflow errors.
    ///  
    ///  The errors counts are for errors that occur on the Bricklet side. All
    ///  Bricks have a similar function that returns the errors on the Brick side.
    /// </summary>
    procedure GetSPITFPErrorCount(out errorCountAckChecksum: longword; out errorCountMessageChecksum: longword; out errorCountFrame: longword; out errorCountOverflow: longword); virtual;

    /// <summary>
    ///  Sets the bootloader mode and returns the status after the requested
    ///  mode change was instigated.
    ///  
    ///  You can change from bootloader mode to firmware mode and vice versa. A change
    ///  from bootloader mode to firmware mode will only take place if the entry function,
    ///  device identifier and CRC are present and correct.
    ///  
    ///  This function is used by Brick Viewer during flashing. It should not be
    ///  necessary to call it in a normal user program.
    /// </summary>
    function SetBootloaderMode(const mode: byte): byte; virtual;

    /// <summary>
    ///  Returns the current bootloader mode, see <see cref="BrickHAT.TBrickHAT.SetBootloaderMode"/>.
    /// </summary>
    function GetBootloaderMode: byte; virtual;

    /// <summary>
    ///  Sets the firmware pointer for <see cref="BrickHAT.TBrickHAT.WriteFirmware"/>. The pointer has
    ///  to be increased by chunks of size 64. The data is written to flash
    ///  every 4 chunks (which equals to one page of size 256).
    ///  
    ///  This function is used by Brick Viewer during flashing. It should not be
    ///  necessary to call it in a normal user program.
    /// </summary>
    procedure SetWriteFirmwarePointer(const pointer: longword); virtual;

    /// <summary>
    ///  Writes 64 Bytes of firmware at the position as written by
    ///  <see cref="BrickHAT.TBrickHAT.SetWriteFirmwarePointer"/> before. The firmware is written
    ///  to flash every 4 chunks.
    ///  
    ///  You can only write firmware in bootloader mode.
    ///  
    ///  This function is used by Brick Viewer during flashing. It should not be
    ///  necessary to call it in a normal user program.
    /// </summary>
    function WriteFirmware(const data: array of byte): byte; virtual;

    /// <summary>
    ///  Sets the status LED configuration. By default the LED shows
    ///  communication traffic between Brick and Bricklet, it flickers once
    ///  for every 10 received data packets.
    ///  
    ///  You can also turn the LED permanently on/off or show a heartbeat.
    ///  
    ///  If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.
    /// </summary>
    procedure SetStatusLEDConfig(const config: byte); virtual;

    /// <summary>
    ///  Returns the configuration as set by <see cref="BrickHAT.TBrickHAT.SetStatusLEDConfig"/>
    /// </summary>
    function GetStatusLEDConfig: byte; virtual;

    /// <summary>
    ///  Returns the temperature in °C as measured inside the microcontroller. The
    ///  value returned is not the ambient temperature!
    ///  
    ///  The temperature is only proportional to the real temperature and it has bad
    ///  accuracy. Practically it is only useful as an indicator for
    ///  temperature changes.
    /// </summary>
    function GetChipTemperature: smallint; virtual;

    /// <summary>
    ///  Calling this function will reset the Bricklet. All configurations
    ///  will be lost.
    ///  
    ///  After a reset you have to create new device objects,
    ///  calling functions on the existing ones will result in
    ///  undefined behavior!
    /// </summary>
    procedure Reset; virtual;

    /// <summary>
    ///  Writes a new UID into flash. If you want to set a new UID
    ///  you have to decode the Base58 encoded UID string into an
    ///  integer first.
    ///  
    ///  We recommend that you use Brick Viewer to change the UID.
    /// </summary>
    procedure WriteUID(const uid: longword); virtual;

    /// <summary>
    ///  Returns the current UID as an integer. Encode as
    ///  Base58 to get the usual string version.
    /// </summary>
    function ReadUID: longword; virtual;

    /// <summary>
    ///  Returns the UID, the UID where the Bricklet is connected to,
    ///  the position, the hardware and firmware version as well as the
    ///  device identifier.
    ///  
    ///  The position can be 'a', 'b', 'c' or 'd'.
    ///  
    ///  The device identifier numbers can be found :ref:`here &lt;device_identifier&gt;`.
    ///  |device_identifier_constant|
    /// </summary>
    procedure GetIdentity(out uid: string; out connectedUid: string; out position: char; out hardwareVersion: TVersionNumber; out firmwareVersion: TVersionNumber; out deviceIdentifier: word); override;
  end;

implementation

constructor TBrickHAT.Create(const uid__: string; ipcon_: TIPConnection);
begin
  inherited Create(uid__, ipcon_);
  apiVersion[0] := 2;
  apiVersion[1] := 0;
  apiVersion[2] := 0;

  responseExpected[BRICK_HAT_FUNCTION_SET_SLEEP_MODE] := DEVICE_RESPONSE_EXPECTED_FALSE;
  responseExpected[BRICK_HAT_FUNCTION_GET_SLEEP_MODE] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICK_HAT_FUNCTION_SET_BRICKLET_POWER] := DEVICE_RESPONSE_EXPECTED_FALSE;
  responseExpected[BRICK_HAT_FUNCTION_GET_BRICKLET_POWER] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICK_HAT_FUNCTION_GET_VOLTAGES] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICK_HAT_FUNCTION_GET_SPITFP_ERROR_COUNT] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICK_HAT_FUNCTION_SET_BOOTLOADER_MODE] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICK_HAT_FUNCTION_GET_BOOTLOADER_MODE] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICK_HAT_FUNCTION_SET_WRITE_FIRMWARE_POINTER] := DEVICE_RESPONSE_EXPECTED_FALSE;
  responseExpected[BRICK_HAT_FUNCTION_WRITE_FIRMWARE] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICK_HAT_FUNCTION_SET_STATUS_LED_CONFIG] := DEVICE_RESPONSE_EXPECTED_FALSE;
  responseExpected[BRICK_HAT_FUNCTION_GET_STATUS_LED_CONFIG] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICK_HAT_FUNCTION_GET_CHIP_TEMPERATURE] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICK_HAT_FUNCTION_RESET] := DEVICE_RESPONSE_EXPECTED_FALSE;
  responseExpected[BRICK_HAT_FUNCTION_WRITE_UID] := DEVICE_RESPONSE_EXPECTED_FALSE;
  responseExpected[BRICK_HAT_FUNCTION_READ_UID] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICK_HAT_FUNCTION_GET_IDENTITY] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;

end;

procedure TBrickHAT.SetSleepMode(const powerOffDelay: longword; const powerOffDuration: longword; const raspberryPiOff: boolean; const brickletsOff: boolean; const enableSleepIndicator: boolean);
var request: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_SET_SLEEP_MODE, 19);
  LEConvertUInt32To(powerOffDelay, 8, request);
  LEConvertUInt32To(powerOffDuration, 12, request);
  LEConvertBooleanTo(raspberryPiOff, 16, request);
  LEConvertBooleanTo(brickletsOff, 17, request);
  LEConvertBooleanTo(enableSleepIndicator, 18, request);
  SendRequest(request);
end;

procedure TBrickHAT.GetSleepMode(out powerOffDelay: longword; out powerOffDuration: longword; out raspberryPiOff: boolean; out brickletsOff: boolean; out enableSleepIndicator: boolean);
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_GET_SLEEP_MODE, 8);
  response := SendRequest(request);
  powerOffDelay := LEConvertUInt32From(8, response);
  powerOffDuration := LEConvertUInt32From(12, response);
  raspberryPiOff := LEConvertBooleanFrom(16, response);
  brickletsOff := LEConvertBooleanFrom(17, response);
  enableSleepIndicator := LEConvertBooleanFrom(18, response);
end;

procedure TBrickHAT.SetBrickletPower(const brickletPower: boolean);
var request: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_SET_BRICKLET_POWER, 9);
  LEConvertBooleanTo(brickletPower, 8, request);
  SendRequest(request);
end;

function TBrickHAT.GetBrickletPower: boolean;
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_GET_BRICKLET_POWER, 8);
  response := SendRequest(request);
  result := LEConvertBooleanFrom(8, response);
end;

procedure TBrickHAT.GetVoltages(out voltageUSB: word; out voltageDC: word);
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_GET_VOLTAGES, 8);
  response := SendRequest(request);
  voltageUSB := LEConvertUInt16From(8, response);
  voltageDC := LEConvertUInt16From(10, response);
end;

procedure TBrickHAT.GetSPITFPErrorCount(out errorCountAckChecksum: longword; out errorCountMessageChecksum: longword; out errorCountFrame: longword; out errorCountOverflow: longword);
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_GET_SPITFP_ERROR_COUNT, 8);
  response := SendRequest(request);
  errorCountAckChecksum := LEConvertUInt32From(8, response);
  errorCountMessageChecksum := LEConvertUInt32From(12, response);
  errorCountFrame := LEConvertUInt32From(16, response);
  errorCountOverflow := LEConvertUInt32From(20, response);
end;

function TBrickHAT.SetBootloaderMode(const mode: byte): byte;
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_SET_BOOTLOADER_MODE, 9);
  LEConvertUInt8To(mode, 8, request);
  response := SendRequest(request);
  result := LEConvertUInt8From(8, response);
end;

function TBrickHAT.GetBootloaderMode: byte;
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_GET_BOOTLOADER_MODE, 8);
  response := SendRequest(request);
  result := LEConvertUInt8From(8, response);
end;

procedure TBrickHAT.SetWriteFirmwarePointer(const pointer: longword);
var request: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_SET_WRITE_FIRMWARE_POINTER, 12);
  LEConvertUInt32To(pointer, 8, request);
  SendRequest(request);
end;

function TBrickHAT.WriteFirmware(const data: array of byte): byte;
var request, response: TByteArray; i: longint;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_WRITE_FIRMWARE, 72);
  if (Length(data) <> 64) then raise EInvalidParameterException.Create('Data has to be exactly 64 items long');
  for i := 0 to Length(data) - 1 do LEConvertUInt8To(data[i], 8 + (i * 1), request);
  response := SendRequest(request);
  result := LEConvertUInt8From(8, response);
end;

procedure TBrickHAT.SetStatusLEDConfig(const config: byte);
var request: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_SET_STATUS_LED_CONFIG, 9);
  LEConvertUInt8To(config, 8, request);
  SendRequest(request);
end;

function TBrickHAT.GetStatusLEDConfig: byte;
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_GET_STATUS_LED_CONFIG, 8);
  response := SendRequest(request);
  result := LEConvertUInt8From(8, response);
end;

function TBrickHAT.GetChipTemperature: smallint;
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_GET_CHIP_TEMPERATURE, 8);
  response := SendRequest(request);
  result := LEConvertInt16From(8, response);
end;

procedure TBrickHAT.Reset;
var request: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_RESET, 8);
  SendRequest(request);
end;

procedure TBrickHAT.WriteUID(const uid: longword);
var request: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_WRITE_UID, 12);
  LEConvertUInt32To(uid, 8, request);
  SendRequest(request);
end;

function TBrickHAT.ReadUID: longword;
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_READ_UID, 8);
  response := SendRequest(request);
  result := LEConvertUInt32From(8, response);
end;

procedure TBrickHAT.GetIdentity(out uid: string; out connectedUid: string; out position: char; out hardwareVersion: TVersionNumber; out firmwareVersion: TVersionNumber; out deviceIdentifier: word);
var request, response: TByteArray; i: longint;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICK_HAT_FUNCTION_GET_IDENTITY, 8);
  response := SendRequest(request);
  uid := LEConvertStringFrom(8, 8, response);
  connectedUid := LEConvertStringFrom(16, 8, response);
  position := LEConvertCharFrom(24, response);
  for i := 0 to 2 do hardwareVersion[i] := LEConvertUInt8From(25 + (i * 1), response);
  for i := 0 to 2 do firmwareVersion[i] := LEConvertUInt8From(28 + (i * 1), response);
  deviceIdentifier := LEConvertUInt16From(31, response);
end;

end.
